from typing import Dict, Any, Optional
import pandas as pd
from .base import SodecoBase
from .schemas.prestation import (
    PrestationSchema, 
    PrestationEntrySchema, 
    CostSchema,
    PrestationCompletedSchema,
    DeletePrestationSchema
)

class Prestations(SodecoBase):
    """Class for managing prestations in Sodeco."""

    def __init__(self, sodeco):
        super().__init__(sodeco)
        self.url = f"{self.sodeco.base_url}prestations"

    def create(self, payload: Dict[str, Any], debug: bool = False) -> dict:
        """
        Create a prestation entry.
        The payload must adhere to the structure defined by the PrestationSchema.
        
        Args:
            payload: The prestation data to create
            debug: If True, only validate the payload without making the request
        
        Returns:
            dict: Response from the API
            
        Raises:
            ValueError: If the payload is invalid or prestations/costs data doesn't match schema
        """
        # Validate the main prestation data
        if not PrestationSchema.validate_prestations(payload):
            raise ValueError("Invalid prestation data")
            
        # If debug mode, return without making request
        if debug:
            return {"status": "valid"}
            
        # Send the POST request to create the prestation entry
        headers, data = self._prepare_raw_request(payload)
        data = self._make_request_with_polling(
            self.url,
            method='POST',
            headers=headers,
            data=data
        )
        return data

    def delete(self, payload: Dict[str, Any]) -> dict:
        """
        Delete prestations for a worker in a specific month and year.
        
        Args:
            payload: Dictionary containing:
                    - WorkerNumber: The worker number to delete prestations for
                    - Month: The month to delete prestations for (1-12)
                    - Year: The year to delete prestations for
        
        Returns:
            dict: Response from the API
            
        Raises:
            ValueError: If the payload is invalid
        """
        # Validate the payload
        if not DeletePrestationSchema.validate_delete(payload):
            raise ValueError("Invalid delete prestation data")
            
        url = f"{self.url}/{payload['WorkerNumber']}/{payload['Year']}/{payload['Month']:02d}"
        return self._make_request_with_polling(url, method='DELETE')

    def complete(self, payload: Dict[str, Any]) -> dict:
        """
        Mark prestations as completed for processing in the system.
        
        Args:
            payload: Dictionary containing:
                    - Month: The month to mark as completed (1-12)
                    - Year: The year to mark as completed
                    - Correction: Optional flag to indicate if this is a correction ('Y' or 'N')
        
        Returns:
            dict: Response from the API
            
        Raises:
            ValueError: If the payload is invalid
        """
        # Validate the payload
        if not PrestationCompletedSchema.validate_completed(payload):
            raise ValueError("Invalid completed prestation data")
            
        # Send the POST request to mark prestations as completed
        headers, data = self._prepare_raw_request(payload)
        data = self._make_request_with_polling(
            f"{self.url}/completed",
            method='POST',
            headers=headers,
            data=data
        )
        return data
